/*
 * Copyright (C) 2001, 2002 The Mir-coders group
 *
 * This file is part of Mir.
 *
 * Mir is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Mir is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Mir; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * In addition, as a special exception, The Mir-coders gives permission to link
 * the code of this program with  any library licensed under the Apache Software License,
 * The Sun (tm) Java Advanced Imaging library (JAI), The Sun JIMI library
 * (or with modified versions of the above that use the same license as the above),
 * and distribute linked combinations including the two.  You must obey the
 * GNU General Public License in all respects for all of the code used other than
 * the above mentioned libraries.  If you modify this file, you may extend this
 * exception to your version of the file, but you are not obligated to do so.
 * If you do not wish to do so, delete this exception statement from your version.
 */
package mircoders.abuse;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import mir.entity.Entity;
import mir.session.Request;
import mir.util.StringRoutines;

/**
 * A ip-based throttling filter.
 *
 * <p>
 * Expressions have the form <time in minutes>:<posting limit>
 */
public class ThrottleFilter extends AbstractFilterType {

  public ThrottleFilter(String aName) {
    super(aName);
  }

  public FilterInstance constructFilterInstance(String anExpression) throws AbuseExc {
    List parts = StringRoutines.splitString(anExpression.trim(), ":");

    if (parts.size()==2) {
      try {
        final int interval = Integer.parseInt((String) parts.get(0));
        final int limit = Integer.parseInt((String) parts.get(1));

        if (interval<1) {
          throw new AbuseExc("Interval should be larger than 0");
        }
        if (limit<1) {
          throw new AbuseExc("Limit should be larger than 0");
        }
        final ThrottleManager manager = new ThrottleManager(interval*1000*60);

        return new FilterInstance() {
          public boolean test(Entity anEntity, Request aRequest) {
            manager.addMessage(aRequest.getHeader("ip"));
            if (manager.count(aRequest.getHeader("ip")) > limit) {
              return true;
            }

            return false;
          }

          public String status() {
            return Integer.toString(manager.messages.size()) + " messages";
          }
        };
      }
      catch (AbuseExc e) {
        throw e;
      }
      catch (Throwable t) {
        throw new AbuseExc("Invalid expression: <interval>:<limit> expected");
      }
    }
		throw new AbuseExc("Invalid expression: <interval>:<limit> expected");
  }

  private class ThrottleManager {
    private long horizon;
    private List messages;

    public ThrottleManager(long aHorizon) {
      horizon = aHorizon;

      messages = new ArrayList();
    }

    public void flush() {
      long limit = System.currentTimeMillis() - horizon;

      while (messages.size()>0 && ((Message) messages.get(0)).getTime()<=limit) {
        messages.remove(0);
      }
    }

    public void addMessage(String anIp) {
      Message newMessage = new Message(anIp, System.currentTimeMillis());
      messages.add(newMessage);
    }

    public int count(String anIp) {
      flush();

      int result = 0;
      Iterator i = messages.iterator();
      while (i.hasNext()) {
        Message message = (Message) i.next();
        if (message.getIp().equals(anIp)) {
          result = result + 1;
        }
      }

      return result;
    }

    private class Message {
      private String ip;
      private long time;

      public Message(String anIp, long aTime) {
        ip = anIp;
        time = aTime;
      }

      public String getIp() {
        return ip;
      }

      public long getTime() {
        return time;
      }
    }
  }
}
